#
# Copyright (c) 2025, NVIDIA CORPORATION. All rights reserved.
#

from aistore.sdk.etl.webserver.utils import serialize_class, deserialize_class

__all__ = [
    "serialize_class",
    "deserialize_class",
]
